/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

package enginerds.controller;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.geom.GeneralPath;
import java.awt.image.BufferedImage;
import java.io.File;
import javax.imageio.ImageIO;
import javax.swing.JPanel;

/**
 *
 * @author CoderS_S
 */
public class GaugeArea extends JPanel {
    
    static int BORDER = 2;
    static int gauges = 0;//used by title string;
    static BufferedImage gaugeBack;
    
    int w;//width of gauge
    int h;//height of gauge
    double max;//maximum gauge value
    double radius;//radius of gauge
    double value;
    String title;//Gauge title string
    String unit;//unit only for use with valueShowing = true
    boolean valueShowing;//true shows value, false shows percentage
    Font valueFont;
    
    BufferedImage image;
    Graphics2D imageG;
    
    static {
        try {
            gaugeBack = ImageIO.read(new File("gauge.png"));
        } catch (Exception e) {
            gaugeBack = new BufferedImage(100, 100, BufferedImage.TYPE_INT_RGB);
        }
    }
    
    public GaugeArea(int w, int h, double max) {
        this(w, h, max, "Gauge_" + (++gauges), false, "");
    }
    
    public GaugeArea(int w, int h, double max, String title, boolean valueShowing, String unit) {
        this.w = w;
        this.h = h;
        this.max = max;
        this.valueShowing = valueShowing;
        this.unit = unit;
        radius = Math.min(w, h) / 2.0 - 10;
        value = 0;
        
        //set the gauge title string
        this.title = title;

        //set up BufferedImage
        image = new BufferedImage(w, h, BufferedImage.TYPE_INT_RGB);
        
        //set up BufferedImage graphics
        imageG = image.createGraphics();

        try {
            valueFont = Font.createFont(Font.TRUETYPE_FONT, new File("digital3.ttf"))
                    .deriveFont((float)(h * 0.125f));
        } catch (Exception e) {
            e.printStackTrace();
            valueFont = new Font("Arial", Font.PLAIN, (int)(h * 0.125));
        }
    }
    
    public void paint(Graphics g) {
        double angle = (Math.PI * 0.75) +
                            ((Math.PI * 1.5) *
                                (Math.max(Math.min(value, max), 0) / max)
                            );
        imageG.setColor(Controller.BORDER_COLOR);
        imageG.fillRect(0, 0, w, h);
        imageG.drawImage(gaugeBack,
                BORDER, BORDER, w - BORDER, h - BORDER,
                0, 0, gaugeBack.getWidth(), gaugeBack.getHeight(),
                null);
        
        //draw the title
        imageG.setColor(Color.white);
        Font titleFont = new Font("Arial", Font.BOLD, 20);
        imageG.setFont(titleFont);
        FontMetrics metrics = imageG.getFontMetrics();
        int titleWidth = metrics.stringWidth(title);
        imageG.drawString(title, (w - titleWidth) / 2, (int)(h * 0.4));
        
        //draw the value and percentage
        imageG.setColor(Color.white);
        //lblContent.setFont(myFont.deriveFont((float)(h - TITLE_HEIGHT) * 0.5f));
        imageG.setFont(valueFont);
        metrics = imageG.getFontMetrics();
 
        if (valueShowing) {
            int valueWidth = metrics.stringWidth(Integer.toString((int)value) + unit);
            imageG.drawString(Integer.toString((int)value) + " " + unit,
                    (w - valueWidth) / 2, (int)(h * 0.75));
        } else {
            int percentageWidth = metrics.stringWidth((int)(100 * value / max) + "%");
            imageG.drawString((int)(100 * value / max) + "%",
                    (w - percentageWidth) / 2, (int)(h * 0.75) + 18);
        }
        
        //draw the needle
        imageG.setColor(
                new Color(
                        (int)Math.max(Math.min(256 * (2 * (0.0 + (value / max))), 255), 0),
                        (int)Math.max(Math.min(256 * (2 * (1.0 - (value / max))), 255), 0), 0
                )
        );
        GeneralPath needle = new GeneralPath(GeneralPath.WIND_EVEN_ODD, 3);
        needle.moveTo(w / 2 + (radius * Math.cos(angle)),
                      h / 2 + (radius * Math.sin(angle)));
        needle.lineTo(w / 2 + (25 * Math.cos(angle + (0.9 * Math.PI))),
                      h / 2 + (25 * Math.sin(angle + (0.9 * Math.PI))));
        needle.lineTo(w / 2 + (25 * Math.cos(angle + (1.1 * Math.PI))),
                      h / 2 + (25 * Math.sin(angle + (1.1 * Math.PI))));
        imageG.fill(needle);
        
        g.drawImage(image, 0, 0, w, h, null);        
    }
    
    public void update(double value) {
        this.value = value;
        repaint();
    }
    
}
