/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

package enginerds.controller;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.image.BufferedImage;
import java.util.Vector;
import javax.swing.JPanel;

/**
 *
 * @author CoderS_S
 */
public class GraphArea extends JPanel {
    
    //static int graphs = 0;
    
    //border around the JPanel
    static int BORDER = 2;
    
    //padding around the graph
    static int TOP_PADDING = 20;
    static int BOTTOM_PADDING = 20;
    static int LEFT_PADDING = 50;
    static int RIGHT_PADDING = 10;
    
    int w; //width in pixels of JPanel
    int h; //height in pixels of JPanel
    double totalTime; //maximum total time displayed (milliseconds)

    //BufferedImage back; //buffered image to be rendered
    //Graphics2D backG;
    long time0;// graph time 0 in system milliseconds
    long timeF;
    double timeOffset = 0;
    
    boolean savingPoints = false; //are we saving points after they go off the screen
    boolean acceptingPoints = true; //are we accepting more points, this turns to false when showing results
    
    Vector<Graph> graphs = new Vector<Graph>(3, 3);
    
    public GraphArea(int w, int h, double totalTime) {
        this.w = w;
        this.h = h;
        this.totalTime = totalTime;
        
        this.setBackground(Color.black);
        setLayout(null);
        
        Mouse mouse = new Mouse();
        this.addMouseListener(mouse);
        this.addMouseMotionListener(mouse);
        
        AutoRoller roller = new AutoRoller();
        roller.start();
    }
    
    public int addGraph(String title, double minValue, double maxValue) {
        Graph graph = new Graph(title, minValue, maxValue);
        graphs.add(graph);
        
        return graphs.size() - 1;
    }
    
    public void setUpBoundaries() {
        int individualHeight = (getHeight() - (TOP_PADDING + BOTTOM_PADDING)) / graphs.size();
        
        for (int i = 0; i < graphs.size(); ++i) {
            graphs.get(i).setBounds(
                    LEFT_PADDING, TOP_PADDING + (i * individualHeight),
                    w - (LEFT_PADDING + RIGHT_PADDING), individualHeight);
            this.add(graphs.get(i));
            
            if (i == graphs.size() - 1) graphs.get(i).setHasScroller(true);
        }
    }
    
    public void update(int index, double value) {
        graphs.get(index).update(value);
    }
    
    public void setSavingPoints(boolean saving) {
        if (!saving) {
            //stop saving points means no scroller anymore
            timeOffset = 0;
        }
        savingPoints = saving;
    }
    
    public void setAcceptingPoints(boolean accepting) {
        acceptingPoints = accepting;
    }
    
    public void clearAll() {
        for (int i = 0; i < graphs.size(); ++i) {
            graphs.get(i).clear();
        }
    }
    
    public class Mouse implements MouseListener, MouseMotionListener {

        Point pointDown;
        
        @Override
        public void mouseDragged(MouseEvent me) {
            if (savingPoints && !acceptingPoints && Math.abs(time0 - timeF) > totalTime) {//if all the points cant fit on the screen
                int graphWidth = getWidth() - (LEFT_PADDING + RIGHT_PADDING);
                double dx = me.getX() - pointDown.getX();
                
                //now scale the change in pixels to time and add it to offset
                //tempTimeOffset is used to avoid assigning timeOffset twice
                double tempTimeOffset = timeOffset + ((dx / graphWidth) * totalTime);
                if (tempTimeOffset < 0) timeOffset = 0;
                else if (tempTimeOffset > Math.abs(time0 - timeF) - totalTime)
                    timeOffset = Math.abs(time0 - timeF) - totalTime;
                else timeOffset = tempTimeOffset;
                
                pointDown = new Point(me.getX(), me.getY());
            }
        }

        @Override
        public void mouseMoved(MouseEvent me) {
        }

        @Override
        public void mouseClicked(MouseEvent me) {
            if (!savingPoints) {
                setSavingPoints(true);
            } else if (acceptingPoints) {
                setAcceptingPoints(false);
            } else {
                setSavingPoints(false);
                setAcceptingPoints(true);
            }
        }

        @Override
        public void mousePressed(MouseEvent me) {
            pointDown = new Point(me.getX(), me.getY());
        }

        @Override
        public void mouseReleased(MouseEvent me) {
        }

        @Override
        public void mouseEntered(MouseEvent me) {
        }

        @Override
        public void mouseExited(MouseEvent me) {
        }
        
    }
    
    public class Graph extends JPanel {
        
        BufferedImage back;
        Graphics2D backG;
        
        LinkedListPoint firstPoint; //first point of the linked list shown in the graph
        LinkedListPoint newestPoint; //newest point shown as a small circle to the right of the graph

        boolean hasScroller = false;
        
        String title;
        double minValue;
        double maxValue; //maximum graph value (y)
        
        public Graph(String title, double minValue, double maxValue) {
            this.title = title;
            this.minValue = minValue;
            this.maxValue = maxValue;
        }
        
        public void setHasScroller(boolean hasScroller) {
            this.hasScroller = hasScroller;
        }
        
        public void paint(Graphics g) {
            backG.setColor(Color.black);
            backG.fillRect(0, 0, getWidth(), getHeight());
            backG.setColor(Color.white);
            backG.drawRect(0, 0, getWidth() - 1, getHeight() - 1);

            int lastX = 0;
            int lastY = 0;

            if (newestPoint != null) {
                backG.fillOval(
                    (getWidth() - newestPoint.getGraphX(newestPoint.time)) - 3,
                    (getHeight() - newestPoint.getGraphY()) - 3,
                    6, 6);
            }
            
            if (firstPoint != null) {
                //lets draw the graph if it exists

                lastX = firstPoint.getGraphX(time0 - timeOffset);
                lastY = firstPoint.getGraphY();

                
                for (LinkedListPoint pointI = firstPoint.next; pointI != null; pointI = pointI.next) {
                    
                    int x = pointI.getGraphX(time0 - timeOffset);
                    int y = pointI.getGraphY(); //in pixels;

                    //we will only draw if x is greater than 0
                    //or the lastX is still in the screen
                    if (x > 0 && lastX < getWidth()) {
                        //if the lastX is smaller than 0, than we want to draw a
                        //partial line so we update lastX and lastY
                        if (lastX < 0) {
                            lastY += (((y - lastY) * (0 - lastX)) / (x - lastX));
                            lastX = 0;
                        }

                        //if x is off the graph than we are done drawing
                        if (x > getWidth()) {

                            //we should draw the second partial (and final) line now
                            y += (((lastY - y) * (getWidth() - x)) / (lastX - x));
                            x = getWidth();

                            //now draw the last (partial) line
                            backG.drawLine(
                                    getWidth() - (int)x,
                                    getHeight() - (int)y,
                                    getWidth() - (int)lastX,
                                    getHeight() - (int)lastY);

                            if (!savingPoints) {
                                //if we are not saving points then discard the rest of the list
                                pointI.next = null;
                                timeF = pointI.time;
                            }

                            break;
                        }

                        backG.drawLine(getWidth() - (int)x, getHeight() - (int)y,
                                       getWidth() - (int)lastX, getHeight() - (int)lastY);

                    }
                    lastX = x;
                    lastY = y;
                }

            }
            
            if (hasScroller) {
                if (savingPoints && time0 - timeF > totalTime) {//if all the points cant fit on the screen
                    //draw the scrollbar now
                    int scrollX1 = (int)((timeOffset) * (getWidth() / (double)(time0 - timeF)));
                    int scrollX2 = (int)((timeOffset + totalTime) * (getWidth() / (double)(time0 - timeF)));

                    backG.fillRect(
                            getWidth() - scrollX2, getHeight() - 4,
                            scrollX2 - scrollX1, 4);
                }
            }
            
            backG.drawString(title, 5, 20);

            g.drawImage(back, 0, 0, null);
        }
    
        public void setBounds(int x, int y, int w, int h) {
            super.setBounds(x, y, w, h);
            System.out.println(w + "," + h);
            
            back = new BufferedImage(w, h, BufferedImage.TYPE_INT_RGB);
            backG = back.createGraphics();
        }
        
        public void update(double value) {
            newestPoint = new LinkedListPoint(value, System.currentTimeMillis(), firstPoint);
            if (acceptingPoints) {
                firstPoint = newestPoint;
                
                //if this is the first point then set time final
                if (firstPoint.next == null) timeF = firstPoint.time;
            }
        }
        
        public void clear() {
            if (firstPoint != null) firstPoint.next = null;
        }
    
        public class LinkedListPoint {
            
            double value;
            long time;
            LinkedListPoint next;
            //LinkedListPoint previous;
            
            public LinkedListPoint(double value, long time, LinkedListPoint next) {
                this.value = value;
                this.time = time;
                this.next = next;
                //previous = null;
            }
            
            public int getGraphY() {
                return (int)((value - minValue) / (maxValue - minValue) * getHeight());
            }
            
            public int getGraphX(double time0) {
                return (int)(((time0 - time) / (double)totalTime) * getWidth());
            }
            
        }
    
    }
    
    public class AutoRoller extends Thread {
        public void run() {
            while (true) {
                try {
                    if (acceptingPoints) time0 = System.currentTimeMillis();
                    repaint();
                    Thread.sleep(50);
                } catch (Exception e) {
                    try {
                        Thread.sleep(30000);
                    } catch (InterruptedException ex) {
                        break;
                    }
                }
            }
            System.out.println("Serious graph failure");
        }
    }
    
}
