/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

package enginerds.controller;

import gnu.io.CommPortIdentifier;
import gnu.io.SerialPort;
import gnu.io.SerialPortEvent;
import gnu.io.SerialPortEventListener;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.util.Enumeration;

/**
 *
 * @author CoderS_S
 */
public class SerialReader extends Reader implements SerialPortEventListener {
    
    //sendable commands
    static final char COMMAND_PING = '1';
    static final char COMMAND_PONG = '2';
    static final char COMMAND_VOLUME_ML = '3';
    static final char COMMAND_START_AUTO_TITRATING = '4';
    static final char COMMAND_START_MANUAL_TITRATING = '5';
    static final char COMMAND_MANUAL_STOP = '9';
    
    String port;
    SerialPort serialPort;
    private BufferedReader input;
    /** The output stream to the port */
    private OutputStream output;
    /** Milliseconds to block while waiting for port open */
    private static final int TIME_OUT = 2000;
    /** Default bits per second for COM port. */
    private static final int DATA_RATE = 9600;
    
    boolean initialized;
    boolean confirmationPongReceived;
    
    boolean active;

    public SerialReader(String comPort) {
        this.port = comPort;

        initialized = false;
        confirmationPongReceived = false;
        
        active = true;
        
        SineGenerator siner = new SineGenerator();
        siner.start();

        Initializer initializer = new Initializer();
        initializer.start();
    }
    
    public class Initializer extends Thread {
        public void run() {
            //connect to the board first
            while (active) {
                Controller.console.writeMessage("Connecting to Board...");
                boolean connected = attemptConnection();
                if (connected) {
                    Controller.console.writeMessage("Successfully connected!");
                    break;
                } else {
                    Controller.console.writeMessage("Failed to connect!");
                }
                try {
                    Controller.console.writeMessage("Retrying in five seconds...");
                    Thread.sleep(5000);
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
            
            while (active) {
                Controller.console.writeMessage("Sending Ping...");
                Controller.commandReader.sendCommand(SerialReader.COMMAND_PING);
                try {
                    Controller.console.writeMessage("Waiting for a response...");
                    Thread.sleep(3000);
                    if (confirmationPongReceived) {
                        Controller.console.writeMessage("Ping returned!");
                        break;
                    } else {
                        Controller.console.writeMessage("No response received!");
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
            
            Controller.console.writeMessage("Initialization completed!");
            initialized  = true;
        }
    }
    
    public boolean attemptConnection() {
		CommPortIdentifier portId = null;
		Enumeration portEnum = CommPortIdentifier.getPortIdentifiers();

		//First, Find an instance of serial port as set in PORT_NAMES.
		while (portEnum.hasMoreElements()) {
			CommPortIdentifier currPortId = (CommPortIdentifier) portEnum.nextElement();
                        System.out.println(port);
			if (currPortId.getName().equals(port)) {
				portId = currPortId;
				break;
			}
		}
		if (portId == null) {
                    System.out.println("Could not find com port");
			return false;
		}

		try {
			// open serial port, and use class name for the appName.
			serialPort = (SerialPort) portId.open(this.getClass().getName(),
					TIME_OUT);

			// set port parameters
			serialPort.setSerialPortParams(DATA_RATE,
					SerialPort.DATABITS_8,
					SerialPort.STOPBITS_1,
					SerialPort.PARITY_NONE);

			// open the streams
			input = new BufferedReader(new InputStreamReader(serialPort.getInputStream()));
			output = serialPort.getOutputStream();

			// add event listeners
			serialPort.addEventListener(this);
			serialPort.notifyOnDataAvailable(true);
		} catch (Exception e) {
			System.err.println(e.toString());
                        return false;
		}
                return true;
    }

    @Override
    public void serialEvent(SerialPortEvent spe) {
        if (spe.getEventType() == SerialPortEvent.DATA_AVAILABLE) {
            try {
                String inputLine = input.readLine();
                
                if (inputLine.length() >= 4) {
                    
                    //retrieve the input prefix
                    String prefix = inputLine.substring(0, 4);
                    
                    //retrieve the argString if one exists
                    String argString = "";
                    if (inputLine.length() > 4) argString = inputLine.substring(4);
                    
                    if (prefix.equals("PONG"))
                        confirmationPongReceived = true;
                    
                    if (confirmationPongReceived)
                        receiveCommand(prefix, argString);
                }
                
            } catch (Exception e) {
                System.err.println(e.toString());
            }
        }
    }
    
    public void sendCommand(int command) {
        try {
            output.write(command);
        } catch (Exception e) {
            e.printStackTrace();
            Controller.console.writeMessage("Error sending command! Aborting.");
        }
    }
    
    public void receiveCommand(String prefix, String argString) {
        super.receiveCommand(prefix, argString);
    }
    
    public class SineGenerator extends Thread {
        public void run() {
            double theta = 0;
            double step = Math.PI / 40;
            while (!initialized && active) {
                theta += step;
                if (theta > 2 * Math.PI) theta -= 2 * Math.PI;
                double multiple1 = (Math.sin(theta) + 1.0) / 2.0;
                double multiple2 = (-Math.sin(theta) + 1.0) / 2.0;
                double multiple3 = (-Math.sin(theta));
                double multiple4 = (Math.sin(theta) * Math.sin(theta));
                receiveCommand("LITE", Double.toString(Controller.gaugeLight.max * multiple1));
                receiveCommand("PUMP", Double.toString(Controller.gaugePumpSpeed.max * multiple2));
                receiveCommand("TEMP", Double.toString(37.8 * multiple2));
                receiveCommand("PHPH", Double.toString(7.0 * multiple3 + 7.0));
                receiveCommand("VOLM", Double.toString(100.0 * multiple4));
                
                try {
                    Thread.sleep(50);
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }
    }
    
    public void stop() {
        try {
            if (serialPort != null) serialPort.removeEventListener();
            if (input != null) input.close();
            if (output != null) output.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
        active = false;
    }
    
}
